@@ -6,7 +6,6 @@ from collections import namedtuple
 
 import kubernetes
 import six
-
 from dagster import Array, Field, Noneable, StringSource
 from dagster import __version__ as dagster_version
 from dagster import check
@@ -32,7 +31,7 @@ DAGSTER_PG_PASSWORD_ENV_VAR = "DAGSTER_PG_PASSWORD"
 # We expect the PG secret to have this key.
 #
 # For an example, see:
-# python_modules/libraries/dagster-k8s/helm/dagster/templates/secret-postgres.yaml
+# helm/dagster/templates/secret-postgres.yaml
 DAGSTER_PG_PASSWORD_SECRET_KEY = "postgresql-password"
 
 # Kubernetes Job object names cannot be longer than 63 characters
@@ -204,10 +203,10 @@ class DagsterK8sJobConfig(
             the Job Pod.
         env_config_maps (Optional[List[str]]): A list of custom ConfigMapEnvSource names from which to
             draw environment variables (using ``envFrom``) for the Job. Default: ``[]``. See:
-        https://kubernetes.io/docs/tasks/inject-data-application/define-environment-variable-container/#define-an-environment-variable-for-a-container
+            https://kubernetes.io/docs/tasks/inject-data-application/define-environment-variable-container/#define-an-environment-variable-for-a-container
         env_secrets (Optional[List[str]]): A list of custom Secret names from which to
             draw environment variables (using ``envFrom``) for the Job. Default: ``[]``. See:
-        https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables
+            https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables
 
     """
 
@@ -466,10 +465,16 @@ def construct_dagster_k8s_job(
         config_map=kubernetes.client.V1ConfigMapVolumeSource(name=job_config.instance_config_map),
     )
 
+    # If the user has defined custom labels, remove them from the pod_template_spec_metadata
+    # key and merge them with the dagster labels
+    user_defined_pod_template_labels = user_defined_k8s_config.pod_template_spec_metadata.pop(
+        "labels", {}
+    )
+
     template = kubernetes.client.V1PodTemplateSpec(
         metadata=kubernetes.client.V1ObjectMeta(
             name=pod_name,
-            labels=dagster_labels,
+            labels=merge_dicts(dagster_labels, user_defined_pod_template_labels),
             **user_defined_k8s_config.pod_template_spec_metadata
         ),
         spec=kubernetes.client.V1PodSpec(
