@@ -1,7 +1,7 @@
+import sys
 import weakref
 
 import kubernetes
-
 from dagster import (
     DagsterInvariantViolationError,
     EventMetadataEntry,
@@ -20,6 +20,7 @@ from dagster.core.origin import PipelineGrpcServerOrigin, PipelinePythonOrigin
 from dagster.core.storage.pipeline_run import PipelineRun, PipelineRunStatus
 from dagster.serdes import ConfigurableClass, ConfigurableClassData, serialize_dagster_namedtuple
 from dagster.utils import frozentags, merge_dicts
+from dagster.utils.error import serializable_error_info_from_exc_info
 
 from .job import (
     DagsterK8sJobConfig,
@@ -56,9 +57,10 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
     Args:
         service_account_name (str): The name of the Kubernetes service account under which to run
             the Job.
-        job_image (str): The ``name`` of the image to use for the Job's Dagster container. This
-            image will be run with the command
-            ``dagster-graphql -p startPipelineExecution -v {executionParams}``.
+        job_image (Optional[str]): The ``name`` of the image to use for the Job's Dagster container.
+            This image will be run with the command
+            ``dagster api execute_run_with_structured_logs``.
+            When using user code deployments, the image should not be specified.
         instance_config_map (str): The ``name`` of an existing Volume to mount into the pod in
             order to provide a ConfigMap for the Dagster instance. This Volume should contain a
             ``dagster.yaml`` with appropriate values for run storage, event log storage, etc.
@@ -91,10 +93,10 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             present in this namespace. Default: ``"default"``
         env_config_maps (Optional[List[str]]): A list of custom ConfigMapEnvSource names from which to
             draw environment variables (using ``envFrom``) for the Job. Default: ``[]``. See:
-        https://kubernetes.io/docs/tasks/inject-data-application/define-environment-variable-container/#define-an-environment-variable-for-a-container
+            https://kubernetes.io/docs/tasks/inject-data-application/define-environment-variable-container/#define-an-environment-variable-for-a-container
         env_secrets (Optional[List[str]]): A list of custom Secret names from which to
             draw environment variables (using ``envFrom``) for the Job. Default: ``[]``. See:
-        https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables
+            https://kubernetes.io/docs/tasks/inject-data-application/distribute-credentials-secure/#configure-all-key-value-pairs-in-a-secret-as-container-environment-variables
     """
 
     def __init__(
@@ -112,6 +114,8 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         job_namespace="default",
         env_config_maps=None,
         env_secrets=None,
+        k8s_client_batch_api=None,
+        k8s_client_core_api=None,
     ):
         self._inst_data = check.opt_inst_param(inst_data, "inst_data", ConfigurableClassData)
         self.job_namespace = check.str_param(job_namespace, "job_namespace")
@@ -126,8 +130,8 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             check.opt_str_param(kubeconfig_file, "kubeconfig_file")
             kubernetes.config.load_kube_config(kubeconfig_file)
 
-        self._batch_api = kubernetes.client.BatchV1Api()
-        self._core_api = kubernetes.client.CoreV1Api()
+        self._batch_api = k8s_client_batch_api or kubernetes.client.BatchV1Api()
+        self._core_api = k8s_client_core_api or kubernetes.client.CoreV1Api()
 
         self.job_config = None
         self._job_image = check.opt_str_param(job_image, "job_image")
@@ -155,7 +159,7 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         job_cfg = DagsterK8sJobConfig.config_type()
 
         run_launcher_extra_cfg = {
-            "job_namespace": StringSource,
+            "job_namespace": Field(StringSource, is_required=False, default_value="default"),
             "load_incluster_config": Field(bool, is_required=False, default_value=True),
             "kubeconfig_file": Field(Noneable(str), is_required=False, default_value=None),
         }
@@ -233,7 +237,7 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
         job_name = "dagster-run-{}".format(run.run_id)
         pod_name = job_name
 
-        user_defined_k8s_config = get_user_defined_k8s_config(frozentags(external_pipeline.tags))
+        user_defined_k8s_config = get_user_defined_k8s_config(frozentags(run.tags))
 
         pipeline_origin = None
         job_config = None
@@ -263,7 +267,7 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             repository_name = external_pipeline.repository_handle.repository_name
             pipeline_origin = PipelinePythonOrigin(
                 pipeline_name=external_pipeline.name,
-                repository_origin=repository_location_handle.get_repository_python_origin(
+                repository_origin=repository_location_handle.reload_repository_python_origin(
                     repository_name
                 ),
             )
@@ -294,12 +298,11 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
             EngineEventData(
                 [
                     EventMetadataEntry.text(job_name, "Kubernetes Job name"),
-                    EventMetadataEntry.text(pod_name, "Kubernetes Pod name"),
                     EventMetadataEntry.text(self.job_namespace, "Kubernetes Namespace"),
                     EventMetadataEntry.text(run.run_id, "Run ID"),
                 ]
             ),
-            cls=K8sRunLauncher,
+            cls=self.__class__,
         )
         return run
 
@@ -316,10 +319,51 @@ class K8sRunLauncher(RunLauncher, ConfigurableClass):
 
     def terminate(self, run_id):
         check.str_param(run_id, "run_id")
+        run = self._instance.get_run_by_id(run_id)
 
-        if not self.can_terminate(run_id):
+        if not run:
+            return False
+
+        self._instance.report_engine_event(
+            message="Received pipeline termination request.", pipeline_run=run, cls=self.__class__
+        )
+
+        can_terminate = self.can_terminate(run_id)
+        if not can_terminate:
+            self._instance.report_engine_event(
+                message="Unable to terminate pipeline; can_terminate returned {}".format(
+                    can_terminate
+                ),
+                pipeline_run=run,
+                cls=self.__class__,
+            )
             return False
 
         job_name = get_job_name_from_run_id(run_id)
 
-        return delete_job(job_name=job_name, namespace=self.job_namespace)
+        try:
+            termination_result = delete_job(job_name=job_name, namespace=self.job_namespace)
+            if termination_result:
+                self._instance.report_engine_event(
+                    message="Pipeline was terminated successfully.",
+                    pipeline_run=run,
+                    cls=self.__class__,
+                )
+            else:
+                self._instance.report_engine_event(
+                    message="Pipeline was not terminated successfully; delete_job returned {}".format(
+                        termination_result
+                    ),
+                    pipeline_run=run,
+                    cls=self.__class__,
+                )
+            return termination_result
+        except Exception:  # pylint: disable=broad-except
+            self._instance.report_engine_event(
+                message="Pipeline was not terminated successfully; encountered error in delete_job",
+                pipeline_run=run,
+                engine_event_data=EngineEventData.engine_error(
+                    serializable_error_info_from_exc_info(sys.exc_info())
+                ),
+                cls=self.__class__,
+            )
